/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "isku_easyzone_frame.h"
#include "isku_key_combo_box.h"

#define ISKU_EASYZONE_FRAME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), ISKU_EASYZONE_FRAME_TYPE, IskuEasyzoneFrameClass))
#define IS_ISKU_EASYZONE_FRAME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), ISKU_EASYZONE_FRAME_TYPE))
#define ISKU_EASYZONE_FRAME_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), ISKU_EASYZONE_FRAME_TYPE, IskuEasyzoneFramePrivate))

typedef struct _IskuEasyzoneFrameClass IskuEasyzoneFrameClass;
typedef struct _IskuEasyzoneFramePrivate IskuEasyzoneFramePrivate;

struct _IskuEasyzoneFrame {
	GtkFrame parent;
	IskuEasyzoneFramePrivate *priv;
};

struct _IskuEasyzoneFrameClass {
	GtkFrameClass parent_class;
};

struct _IskuEasyzoneFramePrivate {
	IskuKeyComboBox *buttons[ISKU_KEYS_EASYZONE_NUM];
	guint button_rkp_index[ISKU_KEYS_EASYZONE_NUM];
};

G_DEFINE_TYPE(IskuEasyzoneFrame, isku_easyzone_frame, GTK_TYPE_FRAME);

void isku_easyzone_frame_set_key(IskuEasyzoneFrame *easyzone_frame, guint key_index, IskuRkpMacroKeyInfo *key_info) {
	IskuEasyzoneFramePrivate *priv = easyzone_frame->priv;
	guint i;
	for (i = 0; i < ISKU_KEYS_EASYZONE_NUM; ++i)
		if (priv->button_rkp_index[i] == key_index)
			isku_key_combo_box_set_value_blocked(priv->buttons[i], key_info);
}

typedef struct {
	gchar const * const name;
	guint index;
	guint standard_type;
} ButtonList;

static ButtonList button_list[ISKU_KEYS_EASYZONE_NUM] = {
	{"1", ISKU_KEY_INDEX_1, ISKU_KEY_TYPE_STANDARD_1},
	{"2", ISKU_KEY_INDEX_1 + 1, ISKU_KEY_TYPE_STANDARD_2},
	{"3", ISKU_KEY_INDEX_1 + 2, ISKU_KEY_TYPE_STANDARD_3},
	{"4", ISKU_KEY_INDEX_1 + 3, ISKU_KEY_TYPE_STANDARD_4},
	{"5", ISKU_KEY_INDEX_1 + 4, ISKU_KEY_TYPE_STANDARD_5},
	{"Q", ISKU_KEY_INDEX_1 + 5, ISKU_KEY_TYPE_STANDARD_Q},
	{"W", ISKU_KEY_INDEX_1 + 6, ISKU_KEY_TYPE_STANDARD_W},
	{"E", ISKU_KEY_INDEX_1 + 7, ISKU_KEY_TYPE_STANDARD_E},
	{"R", ISKU_KEY_INDEX_1 + 8, ISKU_KEY_TYPE_STANDARD_R},
	{"T", ISKU_KEY_INDEX_1 + 9, ISKU_KEY_TYPE_STANDARD_T},
	{"A", ISKU_KEY_INDEX_1 + 10, ISKU_KEY_TYPE_STANDARD_A},
	{"S", ISKU_KEY_INDEX_1 + 11, ISKU_KEY_TYPE_STANDARD_S},
	{"D", ISKU_KEY_INDEX_1 + 12, ISKU_KEY_TYPE_STANDARD_D},
	{"F", ISKU_KEY_INDEX_1 + 13, ISKU_KEY_TYPE_STANDARD_F},
	{"G", ISKU_KEY_INDEX_1 + 14, ISKU_KEY_TYPE_STANDARD_G},
	{"Y", ISKU_KEY_INDEX_1 + 15, ISKU_KEY_TYPE_STANDARD_Y},
	{"X", ISKU_KEY_INDEX_1 + 16, ISKU_KEY_TYPE_STANDARD_X},
	{"C", ISKU_KEY_INDEX_1 + 17, ISKU_KEY_TYPE_STANDARD_C},
	{"V", ISKU_KEY_INDEX_1 + 18, ISKU_KEY_TYPE_STANDARD_V},
	{"B", ISKU_KEY_INDEX_1 + 19, ISKU_KEY_TYPE_STANDARD_B},
};

GtkWidget *isku_easyzone_frame_new(void) {
	IskuEasyzoneFrame *frame;

	frame = ISKU_EASYZONE_FRAME(g_object_new(ISKU_EASYZONE_FRAME_TYPE,
			"label", "Easyzone keys",
			NULL));

	return GTK_WIDGET(frame);
}

static void isku_easyzone_frame_init(IskuEasyzoneFrame *frame) {
	IskuEasyzoneFramePrivate *priv = ISKU_EASYZONE_FRAME_GET_PRIVATE(frame);
	GtkWidget *lefttable;
	GtkWidget *righttable;
	GtkWidget *table;
	GtkWidget *label;
	GtkWidget *hbox;
	guint i;
	guint j;
	guint easyzone_num_half;

	frame->priv = priv;

	easyzone_num_half = ISKU_KEYS_EASYZONE_NUM >> 1;

	hbox = gtk_hbox_new(FALSE, 0);
	lefttable = gtk_table_new(easyzone_num_half, 3, FALSE);
	righttable = gtk_table_new(easyzone_num_half, 3, FALSE);

	for (i = 0; i < ISKU_KEYS_EASYZONE_NUM; ++i) {
		if (i < easyzone_num_half) {
			table = lefttable;
			j = i;
		} else {
			table = righttable;
			j = i - easyzone_num_half;
		}

		label = gtk_label_new(NULL);
		priv->button_rkp_index[i] = button_list[i].index;
		priv->buttons[i] = ISKU_KEY_COMBO_BOX(
				isku_key_combo_box_new(GTK_LABEL(label), 0, button_list[i].standard_type));
		gtk_table_attach(GTK_TABLE(table), gtk_label_new(button_list[i].name), 0, 1, j, j + 1, GTK_EXPAND, GTK_EXPAND, 0, 0);
		gtk_table_attach(GTK_TABLE(table), GTK_WIDGET(priv->buttons[i]), 1, 2, j, j + 1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 0, 0);
		gtk_table_attach(GTK_TABLE(table), GTK_WIDGET(label), 2, 3, j, j + 1, GTK_EXPAND, GTK_EXPAND, 0, 0);
	}

	gtk_box_pack_start(GTK_BOX(hbox), lefttable, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), gtk_vseparator_new(), FALSE, FALSE, 10);
	gtk_box_pack_start(GTK_BOX(hbox), righttable, FALSE, FALSE, 0);

	gtk_container_add(GTK_CONTAINER(frame), hbox);
}

static void isku_easyzone_frame_class_init(IskuEasyzoneFrameClass *klass) {
	g_type_class_add_private(klass, sizeof(IskuEasyzoneFramePrivate));
}

void isku_easyzone_frame_set_from_rkp(IskuEasyzoneFrame *buttons_frame, IskuRkp *rkp) {
	IskuEasyzoneFramePrivate *priv = buttons_frame->priv;
	guint i;
	IskuRkpMacroKeyInfo *key_info;

	for (i = 0; i < ISKU_KEYS_EASYZONE_NUM; ++i) {
		key_info = isku_rkp_get_macro_key_info(rkp, priv->button_rkp_index[i]);
		isku_key_combo_box_set_value_blocked(priv->buttons[i], key_info);
	}
}

void isku_easyzone_frame_update_rkp(IskuEasyzoneFrame *buttons_frame, IskuRkp *rkp) {
	IskuEasyzoneFramePrivate *priv = buttons_frame->priv;
	guint i;
	IskuRkpMacroKeyInfo *key_info;

	for (i = 0; i < ISKU_KEYS_EASYZONE_NUM; ++i) {
		key_info = isku_key_combo_box_get_value(priv->buttons[i]);
		isku_rkp_set_macro_key_info(rkp, priv->button_rkp_index[i], key_info);
		isku_rkp_macro_key_info_free(key_info);
	}
}
