/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "isku_light_frame.h"
#include "roccat_hscale.h"

#define ISKU_LIGHT_FRAME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), ISKU_LIGHT_FRAME_TYPE, IskuLightFrameClass))
#define IS_ISKU_LIGHT_FRAME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), ISKU_LIGHT_FRAME_TYPE))
#define ISKU_LIGHT_FRAME_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), ISKU_LIGHT_FRAME_TYPE, IskuLightFramePrivate))

typedef struct _IskuLightFrameClass IskuLightFrameClass;
typedef struct _IskuLightFramePrivate IskuLightFramePrivate;

struct _IskuLightFrame {
	GtkFrame parent;
	IskuLightFramePrivate *priv;
};

struct _IskuLightFrameClass {
	GtkFrameClass parent_class;
};

struct _IskuLightFramePrivate {
	RoccatHScale *brightness;
	RoccatHScale *dimness;
	GtkSpinButton *timeout;
};

G_DEFINE_TYPE(IskuLightFrame, isku_light_frame, GTK_TYPE_FRAME);

void isku_light_frame_set_brightness_blocked(IskuLightFrame *frame, guint new_brightness) {
	roccat_hscale_set_value_blocked(frame->priv->brightness, new_brightness);
}

void isku_light_frame_set_from_rkp(IskuLightFrame *frame, IskuRkp *rkp) {
	IskuLightFramePrivate *priv = frame->priv;

	roccat_hscale_set_value_blocked(priv->brightness, isku_rkp_get_light_brightness(rkp));
	roccat_hscale_set_value_blocked(priv->dimness, isku_rkp_get_light_dimness(rkp));
	gtk_spin_button_set_value(priv->timeout, isku_rkp_get_light_timeout(rkp));
}

void isku_light_frame_update_rkp(IskuLightFrame *frame, IskuRkp *rkp) {
	IskuLightFramePrivate *priv = frame->priv;

	isku_rkp_set_light_brightness(rkp, roccat_hscale_get_value(priv->brightness));
	isku_rkp_set_light_dimness(rkp, roccat_hscale_get_value(priv->dimness));
	isku_rkp_set_light_timeout(rkp, gtk_spin_button_get_value(priv->timeout));
}

GtkWidget *isku_light_frame_new(void) {
	IskuLightFrame *isku_light_frame;

	isku_light_frame = ISKU_LIGHT_FRAME(g_object_new(ISKU_LIGHT_FRAME_TYPE,
			NULL));

	return GTK_WIDGET(isku_light_frame);
}

static void isku_light_frame_init(IskuLightFrame *frame) {
	IskuLightFramePrivate *priv = ISKU_LIGHT_FRAME_GET_PRIVATE(frame);
	frame->priv = priv;
	GtkWidget *table;

	table = gtk_table_new(3, 2, FALSE);
	priv->brightness = ROCCAT_HSCALE(roccat_hscale_new_with_range(ISKU_LIGHT_BRIGHTNESS_MIN, ISKU_LIGHT_BRIGHTNESS_MAX, 1));
	priv->dimness = ROCCAT_HSCALE(roccat_hscale_new_with_range(ISKU_LIGHT_BRIGHTNESS_MIN, ISKU_LIGHT_BRIGHTNESS_MAX, 1));
	priv->timeout = GTK_SPIN_BUTTON(gtk_spin_button_new_with_range(1, G_MAXUINT8, 1));

	gtk_table_attach(GTK_TABLE(table), gtk_label_new("Brightness"), 0, 1, 0, 1, GTK_SHRINK, GTK_EXPAND, 0, 0);
	gtk_table_attach(GTK_TABLE(table), gtk_label_new("Dimness"), 0, 1, 1, 2, GTK_SHRINK, GTK_EXPAND, 0, 0);
	gtk_table_attach(GTK_TABLE(table), gtk_label_new("Timeout"), 0, 1, 2, 3, GTK_SHRINK, GTK_EXPAND, 0, 0);
	gtk_table_attach(GTK_TABLE(table), GTK_WIDGET(priv->brightness), 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 0, 0);
	gtk_table_attach(GTK_TABLE(table), GTK_WIDGET(priv->dimness), 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 0, 0);
	gtk_table_attach(GTK_TABLE(table), GTK_WIDGET(priv->timeout), 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 0, 0);
	gtk_container_add(GTK_CONTAINER(frame), table);

	gtk_frame_set_label(GTK_FRAME(frame), "Light");
}

static void isku_light_frame_class_init(IskuLightFrameClass *klass) {
	g_type_class_add_private(klass, sizeof(IskuLightFramePrivate));
}
