/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "roccat/roccat.h"
#include <sys/wait.h>
#include <errno.h>
#include <unistd.h>

int roccat_fork_terminateable(pid_t *pid) {
	int result, status;
	pid_t mypid;

	mypid = *pid;
	if (mypid > 0) {
		/* killing is only possible if thread is still running */
		result = kill(mypid, SIGTERM);
		if (result == -1) {
			g_debug("kill failed %s", g_strerror(errno));
			return -1;
		}

		waitpid(mypid, &status, 0);
		*pid = 0;
		/* process was killed, do nothing more */
		if (!WIFEXITED(status))
			return 1;

		/* process was already finished, start again */
	}

	mypid = fork();
	switch (mypid) {
	case 0:
		return 0;
		break;
	case -1:
		g_debug("fork failed %s", g_strerror(errno));
		*pid = 0;
		return -1;
		break;
	default:
		*pid = mypid;
		return 1;
	}
}

/* excerpt from http://www.mercymachines.net/unix/faq_54.html */
int roccat_fork2(void) {
	pid_t pid;
	int status;

	if (!(pid = fork())) {

		roccat_secure();

		switch (fork()) {
		case 0: return 0;
		case -1: _exit(errno); /* assumes all errnos are <256 */
		default: _exit(0);
		}
	}

	if (pid < 0 || waitpid(pid, &status, 0) < 0)
		return -1;

	if (WIFEXITED(status))
		if (WEXITSTATUS(status) == 0)
			return 1;
		else
			errno = WEXITSTATUS(status);
	else
		errno = EINTR; /* well, sort of :-) */

	return -1;
}
