/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "roccat/lua.h"

static gchar const * const group_name = "LuaGUI";
static gchar const * const rmp_path_key = "RmpFilePath";

gchar *lua_configuration_dir(void) {
	gchar *base = roccat_configuration_dir();
	gchar *dir = g_build_path("/", base, "lua", NULL);
	g_free(base);
	return dir;
}

gboolean lua_configuration_dir_create_if_needed(GError **error) {
	gchar *dir;
	gboolean retval;

	dir = lua_configuration_dir();
	retval = roccat_create_dir_if_needed(dir, error);
	g_free(dir);
	return retval;
}

static gchar *configuration_path(void) {
	gchar *dir = lua_configuration_dir();
	gchar *path = g_build_filename(dir, "roccat_lua.ini", NULL);
	g_free(dir);
	return path;
}

RoccatKeyFile *lua_configuration_load(void) {
	RoccatKeyFile *config;
	gchar *path;

	path = configuration_path();
	config = roccat_key_file_load(path);
	g_free(path);

	return config;
}

void lua_configuration_free(RoccatKeyFile *config) {
	if (config == NULL)
		return;

	roccat_key_file_free(config);
}

gboolean lua_configuration_save(RoccatKeyFile *config, GError **error) {
	if (config == NULL)
		return TRUE;

	if (!lua_configuration_dir_create_if_needed(error))
		return FALSE;

	return roccat_key_file_save(config, error);
}

gchar *lua_configuration_get_rmp_path(RoccatKeyFile *config) {
	return roccat_key_file_get_string_with_default(config, group_name, rmp_path_key, g_get_home_dir());
}

void lua_configuration_set_rmp_path(RoccatKeyFile *config, gchar const *path) {
	roccat_key_file_set_string(config, group_name, rmp_path_key, path);
}
