/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "luaconfig_key_combo_box.h"
#include "roccat/lua.h"
#include "gtk_roccat_helper.h"

#define LUACONFIG_KEY_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), LUACONFIG_KEY_COMBO_BOX_TYPE, LuaconfigKeyComboBoxClass))
#define IS_LUACONFIG_KEY_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), LUACONFIG_KEY_COMBO_BOX_TYPE))
#define LUACONFIG_KEY_COMBO_BOX_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj), LUACONFIG_KEY_COMBO_BOX_TYPE, LuaconfigKeyComboBoxClass))

typedef struct _LuaconfigKeyComboBoxClass LuaconfigKeyComboBoxClass;

struct _LuaconfigKeyComboBox {
	GtkComboBox parent;
};

struct _LuaconfigKeyComboBoxClass {
	GtkComboBoxClass parent_class;
	GtkTreeModel *model;
};

enum {
	TITLE_COLUMN,
	TYPE_COLUMN,
	N_COLUMNS
};

G_DEFINE_TYPE(LuaconfigKeyComboBox, luaconfig_key_combo_box, GTK_TYPE_COMBO_BOX);

GtkWidget *luaconfig_key_combo_box_new(void) {
	LuaconfigKeyComboBox *key_combo_box;
	LuaconfigKeyComboBoxClass *key_combo_box_class;

	key_combo_box = LUACONFIG_KEY_COMBO_BOX(g_object_new(LUACONFIG_KEY_COMBO_BOX_TYPE,
			NULL));

	key_combo_box_class = LUACONFIG_KEY_COMBO_BOX_GET_CLASS(key_combo_box);
	gtk_combo_box_set_model(GTK_COMBO_BOX(key_combo_box), key_combo_box_class->model);

	return GTK_WIDGET(key_combo_box);
}

static void luaconfig_key_combo_box_init(LuaconfigKeyComboBox *key_combo_box) {
	GtkCellRenderer *cell;

	cell = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(key_combo_box), cell, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(key_combo_box), cell, "text", TITLE_COLUMN, NULL);
}

static GtkTreeModel *button_info_model_new(void) {
	GtkTreeStore *store;
	GtkTreeIter toplevel;

	store = gtk_tree_store_new(N_COLUMNS, G_TYPE_STRING, G_TYPE_INT);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Click", TYPE_COLUMN, LUA_KEY_TYPE_CLICK, -1);
	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Menu", TYPE_COLUMN, LUA_KEY_TYPE_MENU, -1);
	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Universal scroll", TYPE_COLUMN, LUA_KEY_TYPE_UNIVERSAL_SCROLL, -1);
	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "IE Forward", TYPE_COLUMN, LUA_KEY_TYPE_IE_FORWARD, -1);
	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "IE Backward", TYPE_COLUMN, LUA_KEY_TYPE_IE_BACKWARD, -1);
	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "CPI Cycle", TYPE_COLUMN, LUA_KEY_TYPE_CPI_CYCLE, -1);
	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Scroll Up", TYPE_COLUMN, LUA_KEY_TYPE_SCROLL_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Scroll Down", TYPE_COLUMN, LUA_KEY_TYPE_SCROLL_DOWN, -1);
	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Disabled", TYPE_COLUMN, LUA_KEY_TYPE_DISABLED, -1);

	return GTK_TREE_MODEL(store);
}

static void luaconfig_key_combo_box_class_init(LuaconfigKeyComboBoxClass *klass) {
	klass->model = button_info_model_new();
}

void luaconfig_key_combo_box_set_value(LuaconfigKeyComboBox *key_combo_box, guint type) {
	GtkTreeIter iter;
	gboolean valid;
	GtkTreeModel *model;

	model = gtk_combo_box_get_model(GTK_COMBO_BOX(key_combo_box));
	valid = gtk_roccat_tree_model_iter_find_int(model, TYPE_COLUMN, type, &iter, NULL);
	if (valid)
		gtk_combo_box_set_active_iter(GTK_COMBO_BOX(key_combo_box), &iter);
}

guint luaconfig_key_combo_box_get_value(LuaconfigKeyComboBox *key_combo_box) {
	GtkTreeIter iter;
	gboolean valid;
	guint value = 0;
	GtkTreeModel *model;

	valid = gtk_combo_box_get_active_iter(GTK_COMBO_BOX(key_combo_box), &iter);
	if (valid) {
		model = gtk_combo_box_get_model(GTK_COMBO_BOX(key_combo_box));
		gtk_tree_model_get(model, &iter, TYPE_COLUMN, &value, -1);
	}
	return value;
}
